Package: classmap
Type: Package
Title: Visualizing Classification Results
Date: 2026-04-28
Version: 1.2.7
Authors@R: c(
            person(given="Jakob",family="Raymaekers", role = c("aut","cre"),
            email="jakob.raymaekers@kuleuven.be"),
            person(given="Peter",family="Rousseeuw", role = c("aut")) 
             )
Author: Jakob Raymaekers [aut, cre],
  Peter Rousseeuw [aut]
Depends: R (>= 4.1.0)
Suggests: knitr, reshape2, svd, rpart.plot, nnet, robCompositions,
        rmarkdown, fairml, robustHD, vioplot
Imports: stats, graphics, ggplot2, robustbase, e1071, cellWise,
        cluster, kernlab, gridExtra, rpart, randomForest, scales,
        lpSolve, diptest
Maintainer: Jakob Raymaekers <jakob.raymaekers@kuleuven.be>
Description: Tools to visualize the results of a classification or a regression.
    The graphical displays include stacked plots, silhouette plots, quasi residual plots, class maps, predictions plots, and predictions correlation plots.
    Implements the techniques described and illustrated in Raymaekers J., Rousseeuw P.J., Hubert M. (2022). Class maps for visualizing classification results. \emph{Technometrics}, 64(2), 151–165. <doi:10.1080/00401706.2021.1927849>
 (open access), Raymaekers J., Rousseeuw P.J.(2022). Silhouettes and quasi residual plots for neural nets and tree-based classifiers. \emph{Journal of Computational and Graphical Statistics}, 31(4), 1332–1343. <doi:10.1080/10618600.2022.2050249>, and Rousseeuw, P.J. (2026). Explainable Linear and Generalized Linear Models by the Predictions Plot. The American Statistician, 80, 157-163, <doi:10.1080/00031305.2025.2539235> (open access), and Montalcini, C., Rousseeuw, P.J. (2025). The bixplot: A variation on the boxplot suited for bimodal data, <doi:10.48550/arXiv.2510.09276> (open access).
 Examples can be found in the vignettes:
    "Discriminant_analysis_examples","K_nearest_neighbors_examples",
    "Support_vector_machine_examples", "Rpart_examples", "Random_forest_examples",
    "Neural_net_examples", "predsplot_examples", and "bixplot_examples".
URL: <doi:10.1080/00401706.2021.1927849>,
        <doi:10.1080/10618600.2022.2050249>,
        <doi:10.1080/00031305.2025.2539235>,
        <doi:10.48550/arXiv.2510.09276>
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.1.2
NeedsCompilation: no
Packaged: 2026-04-29 11:09:36 UTC; Jakob Raymaekers
Repository: CRAN
Date/Publication: 2026-04-29 11:40:08 UTC
Built: R 4.6.0; ; 2026-05-01 21:19:34 UTC; unix
