
import("ggplot2")
import("gridExtra")
import("rpart")
importFrom("e1071", "svm")
importFrom("robustbase", "covMcd")
importFrom("randomForest", "randomForest", "importance")
importFrom("cellWise", "transfo", "truncPC")
importFrom("cluster", "daisy", "pam", "silhouette")
importFrom("grDevices", "adjustcolor", "rainbow", "colorRampPalette")
importFrom("graphics", "abline", "axis", "par", "rect", "title", "lines",
           "plot", "layout", "image", "mtext", "hist","strwidth", "curve",
           "points", "polygon", "box", "segments")
importFrom("stats", "coef", "cov", "dist", "mad", "mahalanobis",
               "median", "pnorm", "prcomp", "qnorm", "sd", "predict",
           "as.dist", "loess", "quantile", "na.omit", "terms.formula",
           "na.pass", "cor", "as.dendrogram", "hclust", "reorder", "order.dendrogram",
            "model.matrix", "approx", "density", "fivenum")
importFrom("kernlab", "stringdot", "kernelMatrix")
importFrom("scales", "breaks_extended")
importFrom("diptest", "dip.test")
importFrom("lpSolve", "lp.transport")


export(makeKernel)
export(makeFV)

export(vcr.da.train)
export(vcr.da.newdata)

export(vcr.knn.train)
export(vcr.knn.newdata)

export(vcr.svm.train)
export(vcr.svm.newdata)

export(confmat.vcr)
export(stackedplot)
export(classmap)

export(vcr.rpart.train)
export(vcr.rpart.newdata)

export(vcr.forest.train)
export(vcr.forest.newdata)

export(vcr.neural.train)
export(vcr.neural.newdata)

export(silplot)
export(qresplot)

export(predsplot)
export(predscor)


export(bixplot)
export(pamc1d)
