\name{data_latenc}
\alias{data_latenc}
\docType{data}
\title{
Latenc data
}
\description{
Latency times in seconds of exploratory behavior in fish during three experimental rounds.

}

\usage{data("data_latenc")}

\format{
  A data frame with 40 observations on the following variables.
  \describe{
  \item{round 1}{Latency times for round 1 of the experiment.}
  \item{round 2}{Latency times for round 1 of the experiment.}
  \item{round 3}{Latency times for round 1 of the experiment.}
}
}


\source{
Kerr, Nicky R., and Travis Ingram. "Personality does not predict individual niche variation in a freshwater fish." Behavioral Ecology 32.1 (2021): 159-167.
}

\examples{
data(data_latenc)
str(data_latenc)

# The data are used in:
\dontrun{
vignette("bixplot_examples")
}
}
